# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List

import httpx

from .claim import (
    ClaimResource,
    AsyncClaimResource,
    ClaimResourceWithRawResponse,
    AsyncClaimResourceWithRawResponse,
    ClaimResourceWithStreamingResponse,
    AsyncClaimResourceWithStreamingResponse,
)
from ....._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from ....._utils import maybe_transform, async_maybe_transform
from ....._compat import cached_property
from ....._resource import SyncAPIResource, AsyncAPIResource
from ....._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ....._base_client import make_request_options
from .....types.api.v2 import (
    clearinghouse_search_payers_params,
    clearinghouse_check_eligibility_params,
    clearinghouse_submit_coordination_of_benefits_params,
)
from .....types.api.v2.clearinghouse_list_payers_response import ClearinghouseListPayersResponse
from .....types.api.v2.clearinghouse_search_payers_response import ClearinghouseSearchPayersResponse
from .....types.api.v2.clearinghouse_check_eligibility_response import ClearinghouseCheckEligibilityResponse

__all__ = ["ClearinghouseResource", "AsyncClearinghouseResource"]


class ClearinghouseResource(SyncAPIResource):
    @cached_property
    def claim(self) -> ClaimResource:
        return ClaimResource(self._client)

    @cached_property
    def with_raw_response(self) -> ClearinghouseResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/samplehc/samplehc-python#accessing-raw-response-data-eg-headers
        """
        return ClearinghouseResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ClearinghouseResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/samplehc/samplehc-python#with_streaming_response
        """
        return ClearinghouseResourceWithStreamingResponse(self)

    def check_eligibility(
        self,
        *,
        provider_identifier: str,
        provider_name: str,
        service_type_codes: List[str],
        subscriber_date_of_birth: str,
        subscriber_first_name: str,
        subscriber_last_name: str,
        subscriber_member_id: str,
        trading_partner_service_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ClearinghouseCheckEligibilityResponse:
        """
        Verifies patient eligibility with a specific payer for given services based on
        the provided patient and provider information.

        Args:
          provider_identifier: The provider identifier. This is usually your NPI.

          provider_name: The provider name.

          service_type_codes: The service type codes.

          subscriber_date_of_birth: The date of birth of the subscriber.

          subscriber_first_name: The first name of the subscriber.

          subscriber_last_name: The last name of the subscriber.

          subscriber_member_id: The member ID of the subscriber.

          trading_partner_service_id: The trading partner service ID

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/api/v2/clearinghouse/check-eligibility",
            body=maybe_transform(
                {
                    "provider_identifier": provider_identifier,
                    "provider_name": provider_name,
                    "service_type_codes": service_type_codes,
                    "subscriber_date_of_birth": subscriber_date_of_birth,
                    "subscriber_first_name": subscriber_first_name,
                    "subscriber_last_name": subscriber_last_name,
                    "subscriber_member_id": subscriber_member_id,
                    "trading_partner_service_id": trading_partner_service_id,
                },
                clearinghouse_check_eligibility_params.ClearinghouseCheckEligibilityParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ClearinghouseCheckEligibilityResponse,
        )

    def list_payers(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ClearinghouseListPayersResponse:
        """Lists all payers available for eligibility checks."""
        return self._get(
            "/api/v2/clearinghouse/payers",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ClearinghouseListPayersResponse,
        )

    def search_payers(
        self,
        *,
        query: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ClearinghouseSearchPayersResponse:
        """
        Searches for payers based on the provided search criteria.

        Args:
          query: The search query (e.g. name, ID, etc.) for the payer.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/api/v2/clearinghouse/payers/search",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {"query": query}, clearinghouse_search_payers_params.ClearinghouseSearchPayersParams
                ),
            ),
            cast_to=ClearinghouseSearchPayersResponse,
        )

    def submit_coordination_of_benefits(
        self,
        *,
        dependent_date_of_birth: str,
        dependent_first_name: str,
        dependent_last_name: str,
        encounter_date_of_service: str,
        encounter_service_type_code: str,
        provider_name: str,
        provider_npi: str,
        subscriber_date_of_birth: str,
        subscriber_first_name: str,
        subscriber_last_name: str,
        subscriber_member_id: str,
        trading_partner_service_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> object:
        """
        Fetches coordination of benefits (COB) information for a patient from a
        specified payer, detailing other insurance coverages.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/api/v2/clearinghouse/coordination-of-benefits",
            body=maybe_transform(
                {
                    "dependent_date_of_birth": dependent_date_of_birth,
                    "dependent_first_name": dependent_first_name,
                    "dependent_last_name": dependent_last_name,
                    "encounter_date_of_service": encounter_date_of_service,
                    "encounter_service_type_code": encounter_service_type_code,
                    "provider_name": provider_name,
                    "provider_npi": provider_npi,
                    "subscriber_date_of_birth": subscriber_date_of_birth,
                    "subscriber_first_name": subscriber_first_name,
                    "subscriber_last_name": subscriber_last_name,
                    "subscriber_member_id": subscriber_member_id,
                    "trading_partner_service_id": trading_partner_service_id,
                },
                clearinghouse_submit_coordination_of_benefits_params.ClearinghouseSubmitCoordinationOfBenefitsParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=object,
        )


class AsyncClearinghouseResource(AsyncAPIResource):
    @cached_property
    def claim(self) -> AsyncClaimResource:
        return AsyncClaimResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncClearinghouseResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/samplehc/samplehc-python#accessing-raw-response-data-eg-headers
        """
        return AsyncClearinghouseResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncClearinghouseResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/samplehc/samplehc-python#with_streaming_response
        """
        return AsyncClearinghouseResourceWithStreamingResponse(self)

    async def check_eligibility(
        self,
        *,
        provider_identifier: str,
        provider_name: str,
        service_type_codes: List[str],
        subscriber_date_of_birth: str,
        subscriber_first_name: str,
        subscriber_last_name: str,
        subscriber_member_id: str,
        trading_partner_service_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ClearinghouseCheckEligibilityResponse:
        """
        Verifies patient eligibility with a specific payer for given services based on
        the provided patient and provider information.

        Args:
          provider_identifier: The provider identifier. This is usually your NPI.

          provider_name: The provider name.

          service_type_codes: The service type codes.

          subscriber_date_of_birth: The date of birth of the subscriber.

          subscriber_first_name: The first name of the subscriber.

          subscriber_last_name: The last name of the subscriber.

          subscriber_member_id: The member ID of the subscriber.

          trading_partner_service_id: The trading partner service ID

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/api/v2/clearinghouse/check-eligibility",
            body=await async_maybe_transform(
                {
                    "provider_identifier": provider_identifier,
                    "provider_name": provider_name,
                    "service_type_codes": service_type_codes,
                    "subscriber_date_of_birth": subscriber_date_of_birth,
                    "subscriber_first_name": subscriber_first_name,
                    "subscriber_last_name": subscriber_last_name,
                    "subscriber_member_id": subscriber_member_id,
                    "trading_partner_service_id": trading_partner_service_id,
                },
                clearinghouse_check_eligibility_params.ClearinghouseCheckEligibilityParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ClearinghouseCheckEligibilityResponse,
        )

    async def list_payers(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ClearinghouseListPayersResponse:
        """Lists all payers available for eligibility checks."""
        return await self._get(
            "/api/v2/clearinghouse/payers",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ClearinghouseListPayersResponse,
        )

    async def search_payers(
        self,
        *,
        query: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ClearinghouseSearchPayersResponse:
        """
        Searches for payers based on the provided search criteria.

        Args:
          query: The search query (e.g. name, ID, etc.) for the payer.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/api/v2/clearinghouse/payers/search",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {"query": query}, clearinghouse_search_payers_params.ClearinghouseSearchPayersParams
                ),
            ),
            cast_to=ClearinghouseSearchPayersResponse,
        )

    async def submit_coordination_of_benefits(
        self,
        *,
        dependent_date_of_birth: str,
        dependent_first_name: str,
        dependent_last_name: str,
        encounter_date_of_service: str,
        encounter_service_type_code: str,
        provider_name: str,
        provider_npi: str,
        subscriber_date_of_birth: str,
        subscriber_first_name: str,
        subscriber_last_name: str,
        subscriber_member_id: str,
        trading_partner_service_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> object:
        """
        Fetches coordination of benefits (COB) information for a patient from a
        specified payer, detailing other insurance coverages.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/api/v2/clearinghouse/coordination-of-benefits",
            body=await async_maybe_transform(
                {
                    "dependent_date_of_birth": dependent_date_of_birth,
                    "dependent_first_name": dependent_first_name,
                    "dependent_last_name": dependent_last_name,
                    "encounter_date_of_service": encounter_date_of_service,
                    "encounter_service_type_code": encounter_service_type_code,
                    "provider_name": provider_name,
                    "provider_npi": provider_npi,
                    "subscriber_date_of_birth": subscriber_date_of_birth,
                    "subscriber_first_name": subscriber_first_name,
                    "subscriber_last_name": subscriber_last_name,
                    "subscriber_member_id": subscriber_member_id,
                    "trading_partner_service_id": trading_partner_service_id,
                },
                clearinghouse_submit_coordination_of_benefits_params.ClearinghouseSubmitCoordinationOfBenefitsParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=object,
        )


class ClearinghouseResourceWithRawResponse:
    def __init__(self, clearinghouse: ClearinghouseResource) -> None:
        self._clearinghouse = clearinghouse

        self.check_eligibility = to_raw_response_wrapper(
            clearinghouse.check_eligibility,
        )
        self.list_payers = to_raw_response_wrapper(
            clearinghouse.list_payers,
        )
        self.search_payers = to_raw_response_wrapper(
            clearinghouse.search_payers,
        )
        self.submit_coordination_of_benefits = to_raw_response_wrapper(
            clearinghouse.submit_coordination_of_benefits,
        )

    @cached_property
    def claim(self) -> ClaimResourceWithRawResponse:
        return ClaimResourceWithRawResponse(self._clearinghouse.claim)


class AsyncClearinghouseResourceWithRawResponse:
    def __init__(self, clearinghouse: AsyncClearinghouseResource) -> None:
        self._clearinghouse = clearinghouse

        self.check_eligibility = async_to_raw_response_wrapper(
            clearinghouse.check_eligibility,
        )
        self.list_payers = async_to_raw_response_wrapper(
            clearinghouse.list_payers,
        )
        self.search_payers = async_to_raw_response_wrapper(
            clearinghouse.search_payers,
        )
        self.submit_coordination_of_benefits = async_to_raw_response_wrapper(
            clearinghouse.submit_coordination_of_benefits,
        )

    @cached_property
    def claim(self) -> AsyncClaimResourceWithRawResponse:
        return AsyncClaimResourceWithRawResponse(self._clearinghouse.claim)


class ClearinghouseResourceWithStreamingResponse:
    def __init__(self, clearinghouse: ClearinghouseResource) -> None:
        self._clearinghouse = clearinghouse

        self.check_eligibility = to_streamed_response_wrapper(
            clearinghouse.check_eligibility,
        )
        self.list_payers = to_streamed_response_wrapper(
            clearinghouse.list_payers,
        )
        self.search_payers = to_streamed_response_wrapper(
            clearinghouse.search_payers,
        )
        self.submit_coordination_of_benefits = to_streamed_response_wrapper(
            clearinghouse.submit_coordination_of_benefits,
        )

    @cached_property
    def claim(self) -> ClaimResourceWithStreamingResponse:
        return ClaimResourceWithStreamingResponse(self._clearinghouse.claim)


class AsyncClearinghouseResourceWithStreamingResponse:
    def __init__(self, clearinghouse: AsyncClearinghouseResource) -> None:
        self._clearinghouse = clearinghouse

        self.check_eligibility = async_to_streamed_response_wrapper(
            clearinghouse.check_eligibility,
        )
        self.list_payers = async_to_streamed_response_wrapper(
            clearinghouse.list_payers,
        )
        self.search_payers = async_to_streamed_response_wrapper(
            clearinghouse.search_payers,
        )
        self.submit_coordination_of_benefits = async_to_streamed_response_wrapper(
            clearinghouse.submit_coordination_of_benefits,
        )

    @cached_property
    def claim(self) -> AsyncClaimResourceWithStreamingResponse:
        return AsyncClaimResourceWithStreamingResponse(self._clearinghouse.claim)
